==> xlr-server-sql-25.3.0-512.1109-sources.jar: com/xebialabs/xlrelease/repository/sql/persistence/CategoryPersistence.scala <==
package com.xebialabs.xlrelease.repository.sql.persistence

import com.xebialabs.xlrelease.api.v1.filter.CategoryFilters
import com.xebialabs.xlrelease.api.v1.views.CategoryView
import com.xebialabs.xlrelease.db.sql.SqlBuilder.Dialect
import com.xebialabs.xlrelease.db.sql.transaction.IsTransactional
import com.xebialabs.xlrelease.domain.id.{CiUid, IdCreator}
import com.xebialabs.xlrelease.domain.{Category, Release}
import com.xebialabs.xlrelease.repository.sql.persistence.Schema.{CATEGORIES, RELEASE_CATEGORY_REFS}
import com.xebialabs.xlrelease.repository.sql.persistence.Utils._
import com.xebialabs.xlrelease.repository.sql.query.CategoryQueryBuilder
import com.xebialabs.xlrelease.repository.sql.query.CategoryQueryBuilder.{STMT_CATEGORY_SELECT, catAlias, refAlias}
import com.xebialabs.xlrelease.repository.sql.{SqlRepository, persistence}
import grizzled.slf4j.Logging
import org.springframework.data.domain.{Page, Pageable}
import org.springframework.jdbc.core.{JdbcTemplate, RowMapper}

import scala.jdk.CollectionConverters._

@IsTransactional
class CategoryPersistence(implicit val jdbcTemplate: JdbcTemplate,
                          implicit val dialect: Dialect)
  extends SqlRepository
    with CategoryMapper
    with PersistenceSupport
    with Logging
    with Utils {

  private val STMT_INSERT_CATEGORY =
    s"""INSERT INTO ${CATEGORIES.TABLE} (
       |  ${CATEGORIES.CI_UID},
       |  ${CATEGORIES.CATEGORY},
       |  ${CATEGORIES.ACTIVE}
       | ) VALUES (
       |  :${CATEGORIES.CI_UID},
       |  :${CATEGORIES.CATEGORY},
       |  :${CATEGORIES.ACTIVE}
       | )
       |""".stripMargin

  private def checkTitleLength(title: String): Unit = {
    if (title.length > persistence.INDEXED_VARCHAR_LENGTH) {
      throw new IllegalArgumentException(s"The category title must be ${persistence.INDEXED_VARCHAR_LENGTH} characters or less")
    }
  }

  def create(category: Category): Category = {
    logger.trace(s"creating category $category")
    checkTitleLength(category.getTitle)
    if (category.getCiUid == null) {
      category.setCiUid(IdCreator.generateId)
    }
    val params = Map(
      CATEGORIES.CI_UID -> category.getCiUid,
      CATEGORIES.CATEGORY -> category.getTitle,
      CATEGORIES.ACTIVE -> category.getActive.asInteger
    )

    sqlInsert(STMT_INSERT_CATEGORY, params)

    category
  }

  private val STMT_UPDATE_CATEGORY =
    s"""UPDATE ${CATEGORIES.TABLE}
       |  SET
       |    ${CATEGORIES.CATEGORY} = :${CATEGORIES.CATEGORY},
       |    ${CATEGORIES.ACTIVE} = :${CATEGORIES.ACTIVE}
       |  WHERE ${CATEGORIES.CI_UID} = :${CATEGORIES.CI_UID}
       |""".stripMargin

  def update(category: Category): Category = {
    logger.trace(s"updating category $category")
    checkTitleLength(category.getTitle)
    val params = Map(
      CATEGORIES.CATEGORY -> category.getTitle,
      CATEGORIES.ACTIVE -> category.getActive.asInteger,
      CATEGORIES.CI_UID -> category.getCiUid
    )

    sqlUpdate(STMT_UPDATE_CATEGORY, params, _ => ())
    category
  }

  private val STMT_DELETE_CATEGORY = {
    s"DELETE FROM ${CATEGORIES.TABLE} WHERE ${CATEGORIES.CI_UID} = :${CATEGORIES.CI_UID}"
  }

  def delete(id: CiUid): Unit = {
    logger.trace(s"deleting category with id[$id]")
    sqlExec(STMT_DELETE_CATEGORY, params(CATEGORIES.CI_UID -> id), _.execute())
  }

  private val STMT_FIND_CATEGORY_BY_ID =
    s"""
       |$STMT_CATEGORY_SELECT
       | WHERE $catAlias.${CATEGORIES.CI_UID} = ?
       |""".stripMargin.linesIterator.filter(_.trim.nonEmpty).mkString(System.lineSeparator)

  def findByCiUid(id: CiUid): Option[Category] = {
    findOptional(_.queryForObject(STMT_FIND_CATEGORY_BY_ID, categoryMapper, id))
  }

  private val STMT_FIND_CATEGORY_BY_TITLE =
    s"""
       |$STMT_CATEGORY_SELECT
       | WHERE LOWER($catAlias.${CATEGORIES.CATEGORY}) = ?
       |""".stripMargin.linesIterator.filter(_.trim.nonEmpty).mkString(System.lineSeparator)


  def findByTitle(title: String): Option[Category] = {
    val searchableTitle = Category.sanitizeTitle(title).toLowerCase()
    findOptional(_.queryForObject(STMT_FIND_CATEGORY_BY_TITLE, categoryMapper, searchableTitle))
  }

  private val STMT_FIND_CATEGORIES_BY_TITLE =
    s"""
       |$STMT_CATEGORY_SELECT
       | WHERE LOWER($catAlias.${CATEGORIES.CATEGORY}) IN (:titles)
       |""".stripMargin.linesIterator.filter(_.trim.nonEmpty).mkString(System.lineSeparator)

  def findByTitles(titles: Seq[String]): Seq[Category] = {
    findMany(
      sqlQuery(STMT_FIND_CATEGORIES_BY_TITLE,
        params("titles" -> titles.map(Category.sanitizeTitle(_).toLowerCase()).asJava),
        categoryMapper)
    )
  }

  def findBy(categoryFilters: CategoryFilters, pageable: Pageable): Page[CategoryView] = {
    CategoryQueryBuilder(dialect, namedTemplate)
      .from(categoryFilters)
      .withPageable(pageable)
      .build()
      .execute()
  }

  def decorate(release: Release): Release = {
    if (release.isTemplate) {
      val categories = findTitlesByReleaseCiUid(release.getCiUid).asJava
      release.setCategories(categories)
    }

    release
  }

  private val STMT_FIND_CATEGORIES_BY_RELEASE_UID: String =
    s"""
       |SELECT
       |  $catAlias.${CATEGORIES.CATEGORY}
       | FROM ${CATEGORIES.TABLE} $catAlias
       | JOIN ${RELEASE_CATEGORY_REFS.TABLE} $refAlias ON $catAlias.${CATEGORIES.CI_UID} = $refAlias.${RELEASE_CATEGORY_REFS.targetColumn}
       | WHERE $refAlias.${RELEASE_CATEGORY_REFS.sourceColumn} = :${RELEASE_CATEGORY_REFS.sourceColumn}
       |""".stripMargin

  def findTitlesByReleaseCiUid(releaseUid: CiUid): Set[String] = {
    findMany(
      sqlQuery(STMT_FIND_CATEGORIES_BY_RELEASE_UID,
        Utils.params(RELEASE_CATEGORY_REFS.sourceColumn -> releaseUid),
        rs => rs.getString(CATEGORIES.CATEGORY))
    ).toSet
  }

  private val STMT_FIND_ACTIVE_CATEGORIES_BY_RELEASE_UIDS: String =
    s"""
       |SELECT
       |  $refAlias.${RELEASE_CATEGORY_REFS.sourceColumn},
       |  $catAlias.${CATEGORIES.CATEGORY}
       | FROM ${CATEGORIES.TABLE} $catAlias
       | JOIN ${RELEASE_CATEGORY_REFS.TABLE} $refAlias ON $catAlias.${CATEGORIES.CI_UID} = $refAlias.${RELEASE_CATEGORY_REFS.targetColumn}
       | WHERE
       |  $refAlias.${RELEASE_CATEGORY_REFS.sourceColumn} IN (:${RELEASE_CATEGORY_REFS.sourceColumn})
       | AND $catAlias.${CATEGORIES.ACTIVE} = 1
       |""".stripMargin

  def findActiveCategoryByReleaseCiUids(releaseCiUids: Set[CiUid]): Map[CiUid, Set[String]] = {
    val activeCategories = sqlQuery(
      STMT_FIND_ACTIVE_CATEGORIES_BY_RELEASE_UIDS,
      params(RELEASE_CATEGORY_REFS.sourceColumn -> releaseCiUids.asJava),
      (rs, _) => (CiUid(rs.getLong(RELEASE_CATEGORY_REFS.sourceColumn)), rs.getString(CATEGORIES.CATEGORY))
    ).toSet

    activeCategories.groupMap(_._1)(_._2)
  }
}

trait CategoryMapper {
  val categoryMapper: RowMapper[Category] = (rs, _) => {
    val category: Category = new Category()
    category.setCiUid(rs.getLong(CATEGORIES.CI_UID))
    category.setTitle(rs.getString(CATEGORIES.CATEGORY))
    category.setActive(rs.getInt(CATEGORIES.ACTIVE).asBoolean)
    category
  }
  val categoryCountMapper: RowMapper[CategoryView] = (rs, _) => {
    val categoryView: CategoryView = new CategoryView()
    categoryView.setId(rs.getString(CATEGORIES.CI_UID))
    categoryView.setTitle(rs.getString(CATEGORIES.CATEGORY))
    categoryView.setActive(rs.getInt(CATEGORIES.ACTIVE).asBoolean)
    categoryView.setAssociatedWorkflowsCount(rs.getInt(4))
    categoryView
  }
}

==> xlr-server-sql-25.3.0-513.1502-sources.jar: com/xebialabs/xlrelease/repository/sql/persistence/CategoryPersistence.scala <==
package com.xebialabs.xlrelease.repository.sql.persistence

import com.xebialabs.xlrelease.api.v1.filter.CategoryFilters
import com.xebialabs.xlrelease.api.v1.views.CategoryView
import com.xebialabs.xlrelease.db.sql.SqlBuilder.Dialect
import com.xebialabs.xlrelease.db.sql.transaction.IsTransactional
import com.xebialabs.xlrelease.domain.id.{CiUid, IdCreator}
import com.xebialabs.xlrelease.domain.{Category, Release}
import com.xebialabs.xlrelease.repository.sql.persistence.Schema.{CATEGORIES, RELEASE_CATEGORY_REFS}
import com.xebialabs.xlrelease.repository.sql.persistence.Utils._
import com.xebialabs.xlrelease.repository.sql.query.CategoryQueryBuilder
import com.xebialabs.xlrelease.repository.sql.query.CategoryQueryBuilder.{STMT_CATEGORY_SELECT, catAlias, refAlias}
import com.xebialabs.xlrelease.repository.sql.{SqlRepository, persistence}
import grizzled.slf4j.Logging
import org.springframework.data.domain.{Page, Pageable}
import org.springframework.jdbc.core.{JdbcTemplate, RowMapper}

import scala.jdk.CollectionConverters._

@IsTransactional
class CategoryPersistence(implicit val jdbcTemplate: JdbcTemplate,
                          implicit val dialect: Dialect)
  extends SqlRepository
    with CategoryMapper
    with PersistenceSupport
    with Logging
    with Utils {

  private val STMT_INSERT_CATEGORY =
    s"""INSERT INTO ${CATEGORIES.TABLE} (
       |  ${CATEGORIES.CI_UID},
       |  ${CATEGORIES.CATEGORY},
       |  ${CATEGORIES.ACTIVE}
       | ) VALUES (
       |  :${CATEGORIES.CI_UID},
       |  :${CATEGORIES.CATEGORY},
       |  :${CATEGORIES.ACTIVE}
       | )
       |""".stripMargin

  private def checkTitleLength(title: String): Unit = {
    if (title.length > persistence.INDEXED_VARCHAR_LENGTH) {
      throw new IllegalArgumentException(s"The category title must be ${persistence.INDEXED_VARCHAR_LENGTH} characters or less")
    }
  }

  def create(category: Category): Category = {
    logger.trace(s"creating category $category")
    checkTitleLength(category.getTitle)
    if (category.getCiUid == null) {
      category.setCiUid(IdCreator.generateId)
    }
    val params = Map(
      CATEGORIES.CI_UID -> category.getCiUid,
      CATEGORIES.CATEGORY -> category.getTitle,
      CATEGORIES.ACTIVE -> category.getActive.asInteger
    )

    sqlInsert(STMT_INSERT_CATEGORY, params)

    category
  }

  private val STMT_UPDATE_CATEGORY =
    s"""UPDATE ${CATEGORIES.TABLE}
       |  SET
       |    ${CATEGORIES.CATEGORY} = :${CATEGORIES.CATEGORY},
       |    ${CATEGORIES.ACTIVE} = :${CATEGORIES.ACTIVE}
       |  WHERE ${CATEGORIES.CI_UID} = :${CATEGORIES.CI_UID}
       |""".stripMargin

  def update(category: Category): Category = {
    logger.trace(s"updating category $category")
    checkTitleLength(category.getTitle)
    val params = Map(
      CATEGORIES.CATEGORY -> category.getTitle,
      CATEGORIES.ACTIVE -> category.getActive.asInteger,
      CATEGORIES.CI_UID -> category.getCiUid
    )

    sqlUpdate(STMT_UPDATE_CATEGORY, params, _ => ())
    category
  }

  private val STMT_DELETE_CATEGORY = {
    s"DELETE FROM ${CATEGORIES.TABLE} WHERE ${CATEGORIES.CI_UID} = :${CATEGORIES.CI_UID}"
  }

  def delete(id: CiUid): Unit = {
    logger.trace(s"deleting category with id[$id]")
    sqlExec(STMT_DELETE_CATEGORY, params(CATEGORIES.CI_UID -> id), _.execute())
  }

  private val STMT_FIND_CATEGORY_BY_ID =
    s"""
       |$STMT_CATEGORY_SELECT
       | WHERE $catAlias.${CATEGORIES.CI_UID} = ?
       |""".stripMargin.linesIterator.filter(_.trim.nonEmpty).mkString(System.lineSeparator)

  def findByCiUid(id: CiUid): Option[Category] = {
    findOptional(_.queryForObject(STMT_FIND_CATEGORY_BY_ID, categoryMapper, id))
  }

  private val STMT_FIND_CATEGORY_BY_TITLE =
    s"""
       |$STMT_CATEGORY_SELECT
       | WHERE LOWER($catAlias.${CATEGORIES.CATEGORY}) = ?
       |""".stripMargin.linesIterator.filter(_.trim.nonEmpty).mkString(System.lineSeparator)


  def findByTitle(title: String): Option[Category] = {
    val searchableTitle = Category.sanitizeTitle(title).toLowerCase()
    findOptional(_.queryForObject(STMT_FIND_CATEGORY_BY_TITLE, categoryMapper, searchableTitle))
  }

  private val STMT_FIND_CATEGORIES_BY_TITLE =
    s"""
       |$STMT_CATEGORY_SELECT
       | WHERE LOWER($catAlias.${CATEGORIES.CATEGORY}) IN (:titles)
       |""".stripMargin.linesIterator.filter(_.trim.nonEmpty).mkString(System.lineSeparator)

  def findByTitles(titles: Seq[String]): Seq[Category] = {
    findMany(
      sqlQuery(STMT_FIND_CATEGORIES_BY_TITLE,
        params("titles" -> titles.map(Category.sanitizeTitle(_).toLowerCase()).asJava),
        categoryMapper)
    )
  }

  def findBy(categoryFilters: CategoryFilters, pageable: Pageable): Page[CategoryView] = {
    CategoryQueryBuilder(dialect, namedTemplate)
      .from(categoryFilters)
      .withPageable(pageable)
      .build()
      .execute()
  }

  def decorate(release: Release): Release = {
    if (release.isTemplate) {
      val categories = findTitlesByReleaseCiUid(release.getCiUid).asJava
      release.setCategories(categories)
    }

    release
  }

  private val STMT_FIND_CATEGORIES_BY_RELEASE_UID: String =
    s"""
       |SELECT
       |  $catAlias.${CATEGORIES.CATEGORY}
       | FROM ${CATEGORIES.TABLE} $catAlias
       | JOIN ${RELEASE_CATEGORY_REFS.TABLE} $refAlias ON $catAlias.${CATEGORIES.CI_UID} = $refAlias.${RELEASE_CATEGORY_REFS.targetColumn}
       | WHERE $refAlias.${RELEASE_CATEGORY_REFS.sourceColumn} = :${RELEASE_CATEGORY_REFS.sourceColumn}
       |""".stripMargin

  def findTitlesByReleaseCiUid(releaseUid: CiUid): Set[String] = {
    findMany(
      sqlQuery(STMT_FIND_CATEGORIES_BY_RELEASE_UID,
        Utils.params(RELEASE_CATEGORY_REFS.sourceColumn -> releaseUid),
        rs => rs.getString(CATEGORIES.CATEGORY))
    ).toSet
  }

  private val STMT_FIND_ACTIVE_CATEGORIES_BY_RELEASE_UIDS: String =
    s"""
       |SELECT
       |  $refAlias.${RELEASE_CATEGORY_REFS.sourceColumn},
       |  $catAlias.${CATEGORIES.CATEGORY}
       | FROM ${CATEGORIES.TABLE} $catAlias
       | JOIN ${RELEASE_CATEGORY_REFS.TABLE} $refAlias ON $catAlias.${CATEGORIES.CI_UID} = $refAlias.${RELEASE_CATEGORY_REFS.targetColumn}
       | WHERE
       |  $refAlias.${RELEASE_CATEGORY_REFS.sourceColumn} IN (:${RELEASE_CATEGORY_REFS.sourceColumn})
       | AND $catAlias.${CATEGORIES.ACTIVE} = 1
       |""".stripMargin

  def findActiveCategoryByReleaseCiUids(releaseCiUids: Set[CiUid]): Map[CiUid, Set[String]] = {
    val activeCategories = sqlQuery(
      STMT_FIND_ACTIVE_CATEGORIES_BY_RELEASE_UIDS,
      params(RELEASE_CATEGORY_REFS.sourceColumn -> releaseCiUids.asJava),
      (rs, _) => (CiUid(rs.getString(RELEASE_CATEGORY_REFS.sourceColumn)), rs.getString(CATEGORIES.CATEGORY))
    ).toSet

    activeCategories.groupMap(_._1)(_._2)
  }
}

trait CategoryMapper {
  val categoryMapper: RowMapper[Category] = (rs, _) => {
    val category: Category = new Category()
    category.setCiUid(rs.getString(CATEGORIES.CI_UID))
    category.setTitle(rs.getString(CATEGORIES.CATEGORY))
    category.setActive(rs.getInt(CATEGORIES.ACTIVE).asBoolean)
    category
  }
  val categoryCountMapper: RowMapper[CategoryView] = (rs, _) => {
    val categoryView: CategoryView = new CategoryView()
    categoryView.setId(rs.getString(CATEGORIES.CI_UID))
    categoryView.setTitle(rs.getString(CATEGORIES.CATEGORY))
    categoryView.setActive(rs.getInt(CATEGORIES.ACTIVE).asBoolean)
    categoryView.setAssociatedWorkflowsCount(rs.getInt(4))
    categoryView
  }
}
